@@ -100,6 +100,8 @@ struct FMTreeModelDetails {
 	gboolean show_hidden_files;
 	gboolean show_backup_files;
 	gboolean show_only_directories;
+
+	GList *highlighted_files;
 };
 
 struct FMTreeModelRoot {
@@ -116,14 +118,17 @@ typedef struct {
 	FMTreeModel *model;
 } DoneLoadingParameters;
 
-static GObjectClass *parent_class;
-
+static void fm_tree_model_tree_model_init (GtkTreeModelIface *iface);
 static void schedule_monitoring_update     (FMTreeModel *model);
 static void destroy_node_without_reporting (FMTreeModel *model,
 					    TreeNode          *node);
 static void report_node_contents_changed   (FMTreeModel *model,
 					    TreeNode          *node);
 
+G_DEFINE_TYPE_WITH_CODE (FMTreeModel, fm_tree_model, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
+						fm_tree_model_tree_model_init));
+
 static GtkTreeModelFlags 
 fm_tree_model_get_flags (GtkTreeModel *tree_model)
 {
@@ -211,7 +216,7 @@ tree_node_destroy (FMTreeModel *model, TreeNode *node)
 	g_assert (node->done_loading_id == 0);
 	g_assert (node->files_added_id == 0);
 	g_assert (node->files_changed_id == 0);
-	object_unref_if_not_NULL (node->directory);
+	nautilus_directory_unref (node->directory);
 
 	g_free (node);
 }
@@ -257,20 +262,37 @@ get_menu_icon (GIcon *icon)
 }
 
 static GdkPixbuf *
-get_menu_icon_for_file (NautilusFile *file,
+get_menu_icon_for_file (TreeNode *node,
+                        NautilusFile *file,
 			NautilusFileIconFlags flags)
 {
 	NautilusIconInfo *info;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *retval;
+	gboolean highlight;
 	int size;
+	FMTreeModel *model;
 
 	size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
-	
+
 	info = nautilus_file_get_icon (file, size, flags);
-	pixbuf = nautilus_icon_info_get_pixbuf_nodefault_at_size (info, size);
+	retval = nautilus_icon_info_get_pixbuf_nodefault_at_size (info, size);
+	model = node->root->model;
+
+	highlight = (g_list_find_custom (model->details->highlighted_files,
+	                                 file, (GCompareFunc) nautilus_file_compare_location) != NULL);
+
+	if (highlight) {
+		pixbuf = eel_gdk_pixbuf_render (retval, 1, 255, 255, 0, 0);
+
+		if (pixbuf != NULL) {
+			g_object_unref (retval);
+			retval = pixbuf;
+		}
+	}
+
 	g_object_unref (info);
-	
-	return pixbuf;
+
+	return retval;
 }
 
 static GdkPixbuf *
@@ -280,7 +302,7 @@ tree_node_get_pixbuf (TreeNode *node,
 	if (node->parent == NULL) {
 		return get_menu_icon (node->icon);
 	}
-	return get_menu_icon_for_file (node->file, flags);
+	return get_menu_icon_for_file (node, node->file, flags);
 }
 
 static gboolean
@@ -901,7 +923,8 @@ should_show_file (FMTreeModel *model, NautilusFile *file)
 
 	should = nautilus_file_should_show (file,
 					    model->details->show_hidden_files,
-					    model->details->show_backup_files);
+					    model->details->show_backup_files,
+					    TRUE);
 
 	if (should
 	    && model->details->show_only_directories
@@ -1811,6 +1834,51 @@ fm_tree_model_file_get_iter (FMTreeModel *model,
 	return FALSE;
 }
 
+static void
+do_update_node (NautilusFile *file,
+                  FMTreeModel *model)
+{
+	TreeNode *root, *node = NULL;
+
+	for (root = model->details->root_node; root != NULL; root = root->next) {
+		node = get_node_from_file (root->root, file);
+
+		if (node != NULL) {
+			break;
+		}
+	}
+
+	if (node == NULL) {
+		return;
+	}
+
+	update_node (model, node);
+}
+
+void
+fm_tree_model_set_highlight_for_files (FMTreeModel *model,
+                                       GList *files)
+{
+	GList *old_files;
+
+	if (model->details->highlighted_files != NULL) {
+		old_files = model->details->highlighted_files;
+		model->details->highlighted_files = NULL;
+
+		g_list_foreach (old_files,
+		                (GFunc) do_update_node, model);
+
+		nautilus_file_list_free (old_files);
+	}
+
+	if (files != NULL) {
+		model->details->highlighted_files = 
+			nautilus_file_list_copy (files);
+		g_list_foreach (model->details->highlighted_files,
+		                (GFunc) do_update_node, model);
+	}
+}
+
 static void
 fm_tree_model_init (FMTreeModel *model)
 {
@@ -1842,16 +1910,18 @@ fm_tree_model_finalize (GObject *object)
 		g_source_remove (model->details->monitoring_update_idle_id);
 	}
 
+	if (model->details->highlighted_files != NULL) {
+		nautilus_file_list_free (model->details->highlighted_files);
+	}
+
 	g_free (model->details);
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (fm_tree_model_parent_class)->finalize (object);
 }
 
 static void
 fm_tree_model_class_init (FMTreeModelClass *class)
 {
-	parent_class = g_type_class_peek_parent (class);
-
 	G_OBJECT_CLASS (class)->finalize = fm_tree_model_finalize;
 
 	tree_model_signals[ROW_LOADED] =
@@ -1884,35 +1954,4 @@ fm_tree_model_tree_model_init (GtkTreeModelIface *iface)
 	iface->unref_node = fm_tree_model_unref_node;
 }
 
-GType
-fm_tree_model_get_type (void)
-{
-	static GType object_type = 0;
 
-	if (object_type == 0) {
-		const GTypeInfo object_info = {
-			sizeof (FMTreeModelClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) fm_tree_model_class_init,
-			NULL,
-			NULL,
-			sizeof (FMTreeModel),
-			0,
-			(GInstanceInitFunc) fm_tree_model_init,
-		};
-
-		const GInterfaceInfo tree_model_info = {
-			(GInterfaceInitFunc) fm_tree_model_tree_model_init,
-			NULL,
-			NULL
-		};
-
-		object_type = g_type_register_static (G_TYPE_OBJECT, "FMTreeModel", &object_info, 0);
-		g_type_add_interface_static (object_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-	}
-
-	return object_type;
-}
