@@ -36,6 +36,7 @@
 #include "nautilus-location-entry.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window.h"
+#include "nautilus-navigation-window-pane.h"
 #include <eel/eel-accessibility.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -44,8 +45,6 @@
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-private/nautilus-icon-dnd.h>
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <stdio.h>
@@ -127,7 +126,7 @@ drag_data_received_callback (GtkWidget *widget,
 	g_assert (data != NULL);
 	g_assert (callback_data == NULL);
 
-	names = g_uri_list_extract_uris (data->data);
+	names = g_uri_list_extract_uris (gtk_selection_data_get_data (data));
 
 	if (names == NULL || *names == NULL) {
 		g_warning ("No D&D URI's");
@@ -214,7 +213,7 @@ drag_data_get_callback (GtkWidget *widget,
 	case NAUTILUS_DND_URI_LIST:
 	case NAUTILUS_DND_TEXT_PLAIN:
 		gtk_selection_data_set (selection_data,
-					selection_data->target,
+					gtk_selection_data_get_target (selection_data),
 					8, (guchar *) entry_text,
 					eel_strlen (entry_text));
 		break;
@@ -235,6 +234,7 @@ style_set_handler (GtkWidget *widget, GtkStyle *previous_style)
 {
 	PangoLayout *layout;
 	int width, width2;
+	int xpad;
 
 	layout = gtk_label_get_layout (GTK_LABEL(widget));
 
@@ -247,7 +247,10 @@ style_set_handler (GtkWidget *widget, GtkStyle *previous_style)
 	pango_layout_get_pixel_size (layout, &width2, NULL);
 	width = MAX (width, width2);
 
-	width += 2 * GTK_MISC (widget)->xpad;
+	gtk_misc_get_padding (GTK_MISC (widget),
+			      &xpad, NULL);
+
+	width += 2 * xpad;
 
 	gtk_widget_set_size_request (widget, width, -1);
 
@@ -267,10 +270,10 @@ label_button_pressed_callback (GtkWidget             *widget,
 		return FALSE;
 	}
 
-	window = nautilus_location_bar_get_window (widget->parent);
-	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+	window = nautilus_location_bar_get_window (gtk_widget_get_parent (widget));
+	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
 	view = slot->content_view;
-	label = GTK_BIN (widget)->child;
+	label = gtk_bin_get_child (GTK_BIN (widget));
 	/* only pop-up if the URI in the entry matches the displayed location */
 	if (view == NULL ||
 	    strcmp (gtk_label_get_text (GTK_LABEL (label)), LOCATION_LABEL)) {
@@ -282,28 +285,6 @@ label_button_pressed_callback (GtkWidget             *widget,
 	return FALSE;
 }
 
-static int
-get_editable_number_of_chars (GtkEditable *editable)
-{
-	char *text;
-	int length;
-
-	text = gtk_editable_get_chars (editable, 0, -1);
-	length = g_utf8_strlen (text, -1);
-	g_free (text);
-	return length;
-}
-
-static void
-set_position_and_selection_to_end (GtkEditable *editable)
-{
-	int end;
-
-	end = get_editable_number_of_chars (editable);
-	gtk_editable_select_region (editable, end, end);
-	gtk_editable_set_position (editable, end);
-}
-
 static void
 editable_activate_callback (GtkEntry *entry,
 			    gpointer user_data)
@@ -320,9 +301,8 @@ editable_activate_callback (GtkEntry *entry,
 }
 
 static void
-editable_event_after_callback (GtkEntry *entry,
-			       GdkEvent *event,
-			       gpointer user_data)
+editable_changed_callback (GtkEntry *entry,
+			   gpointer user_data)
 {
 	nautilus_location_bar_update_label (NAUTILUS_LOCATION_BAR (user_data));
 }
@@ -417,8 +397,7 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 	event_box = gtk_event_box_new ();
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE);
 	
-	gtk_container_set_border_width (GTK_CONTAINER (event_box),
-					GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER (event_box), 4);
 	label = gtk_label_new (LOCATION_LABEL);
 	gtk_container_add   (GTK_CONTAINER (event_box), label);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
@@ -426,15 +405,14 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 	g_signal_connect (label, "style_set", 
 			  G_CALLBACK (style_set_handler), NULL);
 
-	gtk_box_pack_start (GTK_BOX (hbox), event_box, FALSE, TRUE,
-			    GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (hbox), event_box, FALSE, TRUE, 4);
 
 	entry = nautilus_location_entry_new ();
 	
 	g_signal_connect_object (entry, "activate",
-				 G_CALLBACK (editable_activate_callback), bar, 0);
-	g_signal_connect_object (entry, "event_after",
-				 G_CALLBACK (editable_event_after_callback), bar, G_CONNECT_AFTER);
+				 G_CALLBACK (editable_activate_callback), bar, G_CONNECT_AFTER);
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (editable_changed_callback), bar, 0);
 
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
@@ -470,7 +448,7 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 }
 
 GtkWidget *
-nautilus_location_bar_new (NautilusNavigationWindow *window)
+nautilus_location_bar_new (NautilusNavigationWindowPane *pane)
 {
 	GtkWidget *bar;
 	NautilusLocationBar *location_bar;
@@ -481,7 +459,7 @@ nautilus_location_bar_new (NautilusNavigationWindow *window)
 	/* Clipboard */
 	nautilus_clipboard_set_up_editable
 		(GTK_EDITABLE (location_bar->details->entry),
-		 nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window)),
+		 nautilus_window_get_ui_manager (NAUTILUS_WINDOW (NAUTILUS_WINDOW_PANE(pane)->window)),
 		 TRUE);
 
 	return bar;
@@ -509,9 +487,8 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 		file = g_file_new_for_uri (location);
 		formatted_location = g_file_get_parse_name (file);
 		g_object_unref (file);
-		nautilus_entry_set_text (NAUTILUS_ENTRY (bar->details->entry),
-					 formatted_location);
-		set_position_and_selection_to_end (GTK_EDITABLE (bar->details->entry));
+		nautilus_location_entry_update_current_location (NAUTILUS_LOCATION_ENTRY (bar->details->entry),
+								 formatted_location);
 		g_free (formatted_location);
 	}
 
@@ -567,16 +544,50 @@ nautilus_location_bar_update_label (NautilusLocationBar *bar)
 	GFile *location;
 	GFile *last_location;
 	
+	if (bar->details->last_location == NULL){
+		gtk_label_set_text (GTK_LABEL (bar->details->label), GO_TO_LABEL);
+		nautilus_location_entry_set_secondary_action (NAUTILUS_LOCATION_ENTRY (bar->details->entry), 
+							      NAUTILUS_LOCATION_ENTRY_ACTION_GOTO);
+		return;
+	}
+
 	current_text = gtk_entry_get_text (GTK_ENTRY (bar->details->entry));
 	location = g_file_parse_name (current_text);
 	last_location = g_file_parse_name (bar->details->last_location);
 	
 	if (g_file_equal (last_location, location)) {
 		gtk_label_set_text (GTK_LABEL (bar->details->label), LOCATION_LABEL);
+		nautilus_location_entry_set_secondary_action (NAUTILUS_LOCATION_ENTRY (bar->details->entry), 
+							      NAUTILUS_LOCATION_ENTRY_ACTION_CLEAR);
 	} else {		 
 		gtk_label_set_text (GTK_LABEL (bar->details->label), GO_TO_LABEL);
+		nautilus_location_entry_set_secondary_action (NAUTILUS_LOCATION_ENTRY (bar->details->entry), 
+							      NAUTILUS_LOCATION_ENTRY_ACTION_GOTO);
 	}
 
 	g_object_unref (location);
 	g_object_unref (last_location);
 }
+
+/* change background color based on activity state */
+void
+nautilus_location_bar_set_active(NautilusLocationBar *location_bar, gboolean is_active)
+{
+	if(is_active) {
+		/* reset style to default */
+		gtk_widget_modify_base (GTK_WIDGET (location_bar->details->entry), GTK_STATE_NORMAL, NULL);
+	}
+	else {
+		GtkStyle *style;
+		GdkColor color;
+		style = gtk_widget_get_style (GTK_WIDGET (location_bar->details->entry));
+		color = style->base[GTK_STATE_INSENSITIVE];
+		gtk_widget_modify_base(GTK_WIDGET (location_bar->details->entry), GTK_STATE_NORMAL, &color);
+	}
+}
+
+NautilusEntry *
+nautilus_location_bar_get_entry (NautilusLocationBar *location_bar)
+{
+	return location_bar->details->entry;
+}
