@@ -27,13 +27,7 @@
 #include <glib/gi18n.h>
 #include <eel/eel-gtk-macros.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 
 struct NautilusSearchBarDetails {
 	GtkWidget *entry;
@@ -43,6 +37,7 @@ struct NautilusSearchBarDetails {
 enum {
        ACTIVATE,
        CANCEL,
+       FOCUS_IN,
        LAST_SIGNAL
 }; 
 
@@ -85,7 +80,16 @@ nautilus_search_bar_class_init (NautilusSearchBarClass *class)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
-	
+
+	signals[FOCUS_IN] =
+		g_signal_new ("focus-in",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusSearchBarClass, focus_in),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 	signals[CANCEL] =
 		g_signal_new ("cancel",
 			      G_TYPE_FROM_CLASS (class),
@@ -109,6 +113,15 @@ entry_has_text (NautilusSearchBar *bar)
        return text != NULL && text[0] != '\0';
 }
 
+static void
+entry_icon_release_cb (GtkEntry *entry,
+		       GtkEntryIconPosition position,
+		       GdkEvent *event,
+		       NautilusSearchBar *bar)
+{
+	g_signal_emit_by_name (entry, "activate", 0);
+}
+
 static void
 entry_activate_cb (GtkWidget *entry, NautilusSearchBar *bar)
 {
@@ -117,6 +130,19 @@ entry_activate_cb (GtkWidget *entry, NautilusSearchBar *bar)
        }
 }
 
+static gboolean
+focus_in_event_callback (GtkWidget *widget,
+			 GdkEventFocus *event,
+			 gpointer user_data)
+{
+	NautilusSearchBar *bar;
+
+	bar = NAUTILUS_SEARCH_BAR (user_data);
+
+	g_signal_emit (bar, signals[FOCUS_IN], 0);
+
+	return FALSE;
+}
 
 static void
 nautilus_search_bar_init (NautilusSearchBar *bar)
@@ -146,10 +172,17 @@ nautilus_search_bar_init (NautilusSearchBar *bar)
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
 	bar->details->entry = gtk_entry_new ();
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (bar->details->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       GTK_STOCK_FIND);
 	gtk_box_pack_start (GTK_BOX (hbox), bar->details->entry, TRUE, TRUE, 0);
 
 	g_signal_connect (bar->details->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), bar);
+	g_signal_connect (bar->details->entry, "icon-release",
+			  G_CALLBACK (entry_icon_release_cb), bar);
+	g_signal_connect (bar->details->entry, "focus-in-event",
+			  G_CALLBACK (focus_in_event_callback), bar);
 
 	gtk_widget_show (bar->details->entry);
 }
@@ -162,7 +195,7 @@ nautilus_search_bar_borrow_entry (NautilusSearchBar *bar)
 	bar->details->entry_borrowed = TRUE;
 
 	binding_set = gtk_binding_set_by_class (G_OBJECT_GET_CLASS (bar));
-	gtk_binding_entry_clear	(binding_set, GDK_Escape, 0);
+	gtk_binding_entry_remove (binding_set, GDK_Escape, 0);
 	return bar->details->entry;
 }
 
